#pragma once

#include <vector>
#include <iostream>

class Cell;

/** Component for implementing the Chamber as Composite Design Pattern.
 */
class ChamberComponent {
public:
  /** Components get connected with their parents...
   */
  ChamberComponent(ChamberComponent* parent):
    m_children(), m_parent(parent)
  {}

  /** Virtual Destructor because ...
   *
   *  Takes care of deleting all children.
   */
  virtual ~ChamberComponent()
  {
    for (auto& iterator : m_children) {
      delete iterator;
    }
  }

  /** Getter for ith Child.
   */
  ChamberComponent* getChild(unsigned iChild);

  /** Getter for next child layer object; this can be a neighbours child.
   */
  ChamberComponent* getNextChild(ChamberComponent* currentChild);

  /** Getter for total number of children of this component.
   */
  unsigned getNChild()
  {
    return m_children.size();
  }

  /** Getter for pointer to parent.
   */
  ChamberComponent* getParent()
  {
    return m_parent;
  }

  /** Get first leaf.
   */
  ChamberComponent* deepFirst()
  {
    if (m_children.empty()) {
      return this;
    }
    return m_children[0]->deepFirst();
  }

  /** Get the hight of the Component.
   */
  virtual unsigned getMaxY() const
  {
    unsigned sizeCounter = 0;
    for (auto& childPtr : m_children) {
      sizeCounter += childPtr->getMaxY();
    }
    return sizeCounter;
  }

  /** Get the width of the Component.
   */
  virtual unsigned getMaxX() const
  {
    m_parent->getMaxX();
  }

  /** Print the component by printing all daughter components.
   */
  virtual std::string visualize()
  {
    for (auto childPtr = m_children.rbegin(); childPtr != m_children.rend(); childPtr++) {
      (*childPtr)->visualize();
    }
    return "";
  }

  /** Get the lower corner of the Component in global coordinates.
   */
  virtual unsigned getMyY() const;

  /** Returns cell at the indicated global position.
   *
   *  Returns a nullptr, if the requested cell is outside the Component.
   */
  Cell* getCellAt(unsigned xPosition, unsigned yPosition);

  /** Creating the Cells/leafs in all layers.
   */
  virtual void fillCells()
  {
    for (auto& childPtr : m_children) {
      childPtr->fillCells();
    }
  }

protected:
  std::vector<ChamberComponent*> m_children;
  ChamberComponent* m_parent;

};
